/***
 * Copyright (C) 1996-98. by Miroslav Zagorac <zaga@tirnanog.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "../lib/lib_debug.h"

#include "cave.h"
#include "demo.h"
#include "graphics.h"
#include "keyboard.h"
#include "main.h"
#include "player.h"


/***
 * The format of the demo data is as follows.  The low nybble of each byte indicates
 * the direction that Rockford is to move (0x0 = end of demo, 0x7 = Right, 0xB = Left,
 * 0xD = Down, 0xE = Up, 0xF = no movement).  The high nybble indicates the number of
 * spaces (number of frames) to apply that movement.  The demo finishes when it hits
 * 0x00. So for example, 0xFF means no movement for 15 turns, 0x1E means move up one space,
 * 0x77 means move right 7 spaces, etc.
 */
static unchar demo_data[] =
{
   0xFF, 0xFF, 0x1E, 0x77, 0x2D, 0x97, 0x4F, 0x2D, 0x47, 0x3E, 0x1B, 0x4F, 0x1E, 0xB7, 0x1D, 0x27, 
   0x4F, 0x6D, 0x17, 0x4D, 0x3B, 0x4F, 0x1D, 0x1B, 0x47, 0x3B, 0x4F, 0x4E, 0x5B, 0x3E, 0x5B, 0x4D, 
   0x3B, 0x5F, 0x3E, 0xAB, 0x1E, 0x3B, 0x1D, 0x6B, 0x4D, 0x17, 0x4F, 0x3D, 0x47, 0x4D, 0x4B, 0x2E, 
   0x27, 0x3E, 0xA7, 0xA7, 0x1D, 0x47, 0x1D, 0x47, 0x2D, 0x5F, 0x57, 0x4E, 0x57, 0x6F, 0x1D, 0x00
};


#ifdef ANSI_FUNC

short demo_keyboard (short init_flag)
#else
   
short demo_keyboard (init_flag)
   short init_flag;
#endif
{
   static short position = 0, counter = 0;

   debug (("{%s(%d)", __FUNCTION__, init_flag))

   if (init_flag)
      position = counter = 0;
   else if (demo_data[position])
   {
      static short demo_key[] = { -1, -1, -1, -1, -1, -1, -1, KEY_GAME_RIGHT, -1, -1, -1, KEY_GAME_LEFT, -1, KEY_GAME_DOWN, KEY_GAME_UP, 0 };

      if (++counter > (demo_data[position] >> 4))
      {
         position++;
         counter = 1;
      }

      debug_return ("} == %d", demo_key[demo_data[position] & 0x0f])
   }

   debug_return ("} == %d", 0)
}


#ifdef ANSI_FUNC

void game_demo (void)
#else
   
void game_demo ()
#endif
{
   short room = game_data.start_room, level = game_data.start_level;

   debug (("{%s()", __FUNCTION__))

   game_data.current_player = 0;
   game_data.game_status    = GAME_DEMO_MODE;

   player_data_set (0, 0, 0);
   demo_keyboard (1);
   graph_palette_fade (-4);

   if (game () & GAME_NEXT_CAVE)
      time_bonus ();

   cave_mask_set (CAVE_MASK_INC, INIT_TIME);
   graph_palette_fade (-4);

   game_data.start_room  = room;
   game_data.start_level = level;

   debug_endfunc
}
